package postgres

import (
	"context"
	"database/sql"
	"encoding/json"

	"git.squidspirit.com/squid/blog.git/backend/internal/adapter/repository"
	"git.squidspirit.com/squid/blog.git/backend/internal/adapter/repository/dbdto"
)

type postDBService struct {
	db *sql.DB
}

func NewPostDBService(db *sql.DB) repository.PostDBService {
	return &postDBService{db}
}

func (srv *postDBService) QueryAll() ([]*dbdto.Post, error) {
	ctx := context.Background()

	rows, err := srv.db.QueryContext(ctx, `
		SELECT
			p.*,
			COALESCE(
				jsonb_agg(
					jsonb_build_object(
						'id', l.id,
						'name', l.name,
						'color', l.color
					) ORDER BY pl.label_order ASC
				)
				FILTER (WHERE l.id IS NOT NULL),
				'[]'::jsonb
			) AS labels
		FROM post p
		LEFT JOIN post_label pl ON p.id = pl.post_id
		LEFT JOIN label l ON pl.label_id = l.id
		GROUP BY p.id
		ORDER BY p.created_time DESC
	`)

	if err != nil {
		return nil, err
	}
	defer rows.Close()

	var labelsJSON []byte

	var postDtos []*dbdto.Post
	for rows.Next() {
		var postDto dbdto.Post
		err = rows.Scan(
			&postDto.ID,
			&postDto.Title,
			&postDto.Content,
			&postDto.Description,
			&postDto.PreviewImageURL,
			&postDto.PublishedTime,
			&postDto.CreatedTime,
			&postDto.UpdatedTime,
			&labelsJSON,
		)

		if err != nil {
			return nil, err
		}

		err := json.Unmarshal(labelsJSON, &postDto.Labels)
		if err != nil {
			return nil, err
		}

		postDtos = append(postDtos, &postDto)
	}

	return postDtos, nil
}
